@@ -36,9 +36,6 @@
 using namespace heif;
 
 
-heif::Error heif::Error::Ok(heif_error_Ok);
-
-
 Fraction::Fraction(int32_t num, int32_t den)
 {
   // Reduce resolution of fraction until we are in a safe range.
@@ -61,8 +58,8 @@ Fraction::Fraction(int32_t num, int32_t den)
 
 Fraction::Fraction(uint32_t num, uint32_t den)
 {
-  assert(num <= (uint32_t)std::numeric_limits<int32_t>::max());
-  assert(den <= (uint32_t)std::numeric_limits<int32_t>::max());
+  assert(num <= (uint32_t) std::numeric_limits<int32_t>::max());
+  assert(den <= (uint32_t) std::numeric_limits<int32_t>::max());
 
   *this = Fraction(int32_t(num), int32_t(den));
 }
@@ -1185,11 +1182,11 @@ Error Box_iloc::append_data(heif_item_id item_ID,
   Extent extent;
   extent.data = data;
 
-  if (construction_method==1) {
+  if (construction_method == 1) {
     extent.offset = m_idat_offset;
     extent.length = data.size();
 
-    m_idat_offset += (int)data.size();
+    m_idat_offset += (int) data.size();
   }
 
   m_items[idx].extents.push_back(std::move(extent));
@@ -2212,6 +2209,14 @@ Error Box_ipma::write(StreamWriter& writer) const
 }
 
 
+void Box_ipma::insert_entries_from_other_ipma_box(const Box_ipma& b)
+{
+  m_entries.insert(m_entries.end(),
+                   b.m_entries.begin(),
+                   b.m_entries.end());
+}
+
+
 Error Box_auxC::parse(BitstreamRange& range)
 {
   parse_full_box_header(range);
@@ -2272,6 +2277,18 @@ Error Box_irot::parse(BitstreamRange& range)
 }
 
 
+Error Box_irot::write(StreamWriter& writer) const
+{
+  size_t box_start = reserve_box_header_space(writer);
+
+  writer.write8((uint8_t)(m_rotation / 90));
+
+  prepend_header(writer, box_start);
+
+  return Error::Ok;
+}
+
+
 std::string Box_irot::dump(Indent& indent) const
 {
   std::ostringstream sstr;
@@ -2299,6 +2316,18 @@ Error Box_imir::parse(BitstreamRange& range)
 }
 
 
+Error Box_imir::write(StreamWriter& writer) const
+{
+  size_t box_start = reserve_box_header_space(writer);
+
+  writer.write8(m_axis == MirrorDirection::Horizontal ? 1 : 0);
+
+  prepend_header(writer, box_start);
+
+  return Error::Ok;
+}
+
+
 std::string Box_imir::dump(Indent& indent) const
 {
   std::ostringstream sstr;
@@ -2330,16 +2359,16 @@ Error Box_clap::parse(BitstreamRange& range)
   // Note: in the standard document 14496-12(2015), it says that the offset values should also be unsigned integers,
   // but this is obviously an error. Even the accompanying standard text says that offsets may be negative.
   int32_t horizontal_offset_num = (int32_t) range.read32();
-  uint32_t horizontal_offset_den = (int32_t)range.read32();
+  uint32_t horizontal_offset_den = (int32_t) range.read32();
   int32_t vertical_offset_num = (int32_t) range.read32();
-  uint32_t vertical_offset_den = (int32_t)range.read32();
+  uint32_t vertical_offset_den = (int32_t) range.read32();
 
   if (clean_aperture_width_num > (uint32_t) std::numeric_limits<int32_t>::max() ||
       clean_aperture_width_den > (uint32_t) std::numeric_limits<int32_t>::max() ||
       clean_aperture_height_num > (uint32_t) std::numeric_limits<int32_t>::max() ||
       clean_aperture_height_den > (uint32_t) std::numeric_limits<int32_t>::max() ||
-      horizontal_offset_den > (uint32_t)std::numeric_limits<int32_t>::max() ||
-      vertical_offset_den > (uint32_t)std::numeric_limits<int32_t>::max()) {
+      horizontal_offset_den > (uint32_t) std::numeric_limits<int32_t>::max() ||
+      vertical_offset_den > (uint32_t) std::numeric_limits<int32_t>::max()) {
     return Error(heif_error_Invalid_input,
                  heif_suberror_Invalid_fractional_number,
                  "Exceeded supported value range.");
@@ -2349,8 +2378,8 @@ Error Box_clap::parse(BitstreamRange& range)
                                     clean_aperture_width_den);
   m_clean_aperture_height = Fraction(clean_aperture_height_num,
                                      clean_aperture_height_den);
-  m_horizontal_offset = Fraction(horizontal_offset_num, (int32_t)horizontal_offset_den);
-  m_vertical_offset = Fraction(vertical_offset_num, (int32_t)vertical_offset_den);
+  m_horizontal_offset = Fraction(horizontal_offset_num, (int32_t) horizontal_offset_den);
+  m_vertical_offset = Fraction(vertical_offset_num, (int32_t) vertical_offset_den);
   if (!m_clean_aperture_width.is_valid() || !m_clean_aperture_height.is_valid() ||
       !m_horizontal_offset.is_valid() || !m_vertical_offset.is_valid()) {
     return Error(heif_error_Invalid_input,
