@@ -137,6 +137,21 @@ heif_filetype_result heif_check_filetype(const uint8_t* data, int len)
 }
 
 
+int heif_check_jpeg_filetype(const uint8_t* data, int len)
+{
+  if (len<12 || data==nullptr) {
+    return -1;
+  }
+
+  static uint8_t jpeg_signature[12] = {
+      0xFF, 0xD8, 0xFF, 0xE0, 0x00, 0x10,
+      0x4A, 0x46, 0x49, 0x46, 0x00, 0x01
+  };
+
+  return strncmp((const char*)data, (const char*)jpeg_signature, 12)==0;
+}
+
+
 heif_brand heif_fourcc_to_brand_enum(const char* fourcc)
 {
   if (fourcc==nullptr || !fourcc[0] || !fourcc[1] || !fourcc[2] || !fourcc[3]) {
@@ -764,12 +779,17 @@ int heif_image_handle_get_ispe_height(const struct heif_image_handle* handle)
 
 int heif_image_handle_has_alpha_channel(const struct heif_image_handle* handle)
 {
-  return handle->image->get_alpha_channel() != nullptr;
+  // TODO: for now, also scan the grid tiles for alpha information (issue #708), but depending about
+  // how the discussion about this structure goes forward, we might remove this again.
+
+  return handle->context->has_alpha(handle->image->get_id());   // handle case in issue #708
+  //return handle->image->get_alpha_channel() != nullptr;       // old alpha check that fails on alpha in grid tiles
 }
 
 
 int heif_image_handle_is_premultiplied_alpha(const struct heif_image_handle* handle)
 {
+  // TODO: what about images that have the alpha in the grid tiles (issue #708) ?
   return handle->image->is_premultiplied_alpha();
 }
 
@@ -987,7 +1007,7 @@ void heif_image_handle_release(const struct heif_image_handle* handle)
 }
 
 
-enum heif_colorspace heif_image_get_colorspace(const struct heif_image* img)
+heif_colorspace heif_image_get_colorspace(const struct heif_image* img)
 {
   return img->image->get_colorspace();
 }
@@ -2254,13 +2274,14 @@ int heif_encoder_has_default(struct heif_encoder* encoder,
 
 static void set_default_options(heif_encoding_options& options)
 {
-  options.version = 4;
+  options.version = 5;
 
   options.save_alpha_channel = true;
   options.macOS_compatibility_workaround = true;
   options.save_two_colr_boxes_when_ICC_and_nclx_available = false;
   options.output_nclx_profile = nullptr;
   options.macOS_compatibility_workaround_no_nclx_profile = true;
+  options.image_orientation = heif_orientation_normal;
 }
 
 static void copy_options(heif_encoding_options& options, const heif_encoding_options& input_options)
@@ -2268,6 +2289,8 @@ static void copy_options(heif_encoding_options& options, const heif_encoding_opt
   set_default_options(options);
 
   switch (input_options.version) {
+    case 5:
+      options.image_orientation = input_options.image_orientation;
     case 4:
       options.output_nclx_profile = input_options.output_nclx_profile;
       options.macOS_compatibility_workaround_no_nclx_profile = input_options.macOS_compatibility_workaround_no_nclx_profile;
@@ -2448,7 +2471,17 @@ struct heif_error heif_context_add_XMP_metadata(struct heif_context* ctx,
                                                 const struct heif_image_handle* image_handle,
                                                 const void* data, int size)
 {
-  Error error = ctx->context->add_XMP_metadata(image_handle->image, data, size);
+  return heif_context_add_XMP_metadata2(ctx, image_handle, data, size,
+                                        heif_metadata_compression_off);
+}
+
+
+struct heif_error heif_context_add_XMP_metadata2(struct heif_context* ctx,
+                                                 const struct heif_image_handle* image_handle,
+                                                 const void* data, int size,
+                                                 heif_metadata_compression compression)
+{
+  Error error = ctx->context->add_XMP_metadata(image_handle->image, data, size, compression);
   if (error != Error::Ok) {
     return error.error_struct(ctx->context.get());
   }
@@ -2464,7 +2497,7 @@ struct heif_error heif_context_add_generic_metadata(struct heif_context* ctx,
                                                     const char* item_type, const char* content_type)
 {
   Error error = ctx->context->add_generic_metadata(image_handle->image, data, size,
-                                                   item_type, content_type);
+                                                   item_type, content_type, heif_metadata_compression_off);
   if (error != Error::Ok) {
     return error.error_struct(ctx->context.get());
   }
